<?php

namespace Drupal\user_disk_quota\Form;

use Drupal\Component\Utility\Bytes;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class UserDiskQuotaSettingsForm.
 *
 * @package Drupal\user_disk_quota\Form
 */
class UserDiskQuotaSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'user_disk_quota.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'user_disk_quota_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#tree'] = TRUE;
    $roles = user_roles();
    $config = $this->config('user_disk_quota.settings');
    $description = [];
    $description[] = $this->t('Specify the user quota limit that applies to each role.');
    $description[] = $this->t("Enter a value like @es1, @es2 or @es3 in order to restrict a user role's disk space quota.", [
      '@es1' => '"512" (bytes)',
      '@es2' => '"80 KB" (kilobytes)',
      '@es3' => '"50 MB" (megabytes)',
    ]);
    $description[] = $this->t('Leave the field empty to not set any limit.');
    $form['roles'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Roles'),
      '#description' => implode(' ', $description),
    ];
    foreach ($roles as $role) {
      $default = $config->get('role_' . $role->id());
      if (!empty($default)) {
        $default = format_size($default);
      }
      $form['roles']['role_' . $role->id()] = [
        '#type' => 'textfield',
        '#title' => $this->t("@role's disk quota limit", ['@role' => $role->label()]),
        '#description' => $this->t('Specify the user quota limit that applies to each role.'),
        '#default_value' => $default,
        '#element_validate' => ['_user_disk_quota_form_quota_validate'],
      ];
    }
    $form['quota_by_type'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Quota by File Type'),
    ];
    $types = $config->get('file_types');
    $form['quota_by_type']['user_disk_quota_file_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('File types'),
      '#description' => $this->t("The file types to be considered in the quotas.") . "<br/>" . $this->t("If no file type is chosen, 'Quota by file type' will be disabled and all file types will count for the disk quota."),
      '#options' => [
        'images' => $this->t('Images'),
        'videos' => $this->t('Videos'),
        'documents' => $this->t('Documents'),
      ],
      '#default_value' => array_combine($types, $types),
    ];
    $form['default'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Default values'),
    ];
    $default_percentage = $config->get('warning_percentage');
    $form['default']['user_disk_quota_warning_percentage'] = [
      '#type' => 'number',
      '#title' => $this->t('Warning message'),
      '#default_value' => $default_percentage,
      '#description' => $this->t('Set the default percentage of user disk quota to be filled up before a warning appears to the user.'),
      '#min' => 1,
      '#max' => 100,
      '#step' => 1,
      '#required' => TRUE,
      '#field_suffix' => '%',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('user_disk_quota.settings');

    foreach ($form_state->getValue('roles') as $key => $value) {
      $value = Bytes::toInt($value);
      if ($value == 0) {
        $value = '';
      }
      $config->set($key, Bytes::toInt($value));
    }

    // If no types are specified quota by type is inactive.
    $quota_by_type = array_filter($form_state->getValue(['quota_by_type', 'user_disk_quota_file_types']));
    $config->set('file_types', array_keys($quota_by_type));
    $config->set('warning_percentage', $form_state->getValue(['default', 'user_disk_quota_warning_percentage']));
    $config->save();

    parent::submitForm($form, $form_state);
  }

}
